@@ -13,10 +13,12 @@ class ObjectInfo(ABC):
     MISSION_ID_TESS = "TIC"
     initial_detrend_period = None
     initial_mask = None
+    star_info = None
 
-    def __init__(self, initial_mask=None, initial_detrend_period=None):
+    def __init__(self, initial_mask=None, initial_detrend_period=None, star_info=None):
         self.initial_mask = initial_mask
         self.initial_detrend_period = initial_detrend_period
+        self.star_info = star_info
 
     @abstractmethod
     def sherlock_id(self):
